#pragma once

#include <Framework/Math.h>
#include <stack>
class BoatyBase;

#define COLLISION_MASK_PLAYER 1
#define COLLISION_MASK_NPC 2

struct HealthInfo
{
	float h;
	float currentHealth;
	ivec2 damageLocation;
	int regenCooldown;
};

class BoatyBase
{
public:
	virtual ~BoatyBase() {}

	std::function<bool(BoatyBase*)> collisionCallback;

	virtual void damageCallback(float h, HealthInfo info) {}
	virtual void regenCallback(float h, HealthInfo info) {}

	float getHealth() const 
	{ 
		return m_health; 
	}

	void doDamage(float h, vec2 pos)
	{
		m_health -= h;
		m_health = glm::clamp(m_health, 0.0f, 1.0f);

		HealthInfo info;
		info.h = h;
		info.currentHealth = m_health;
		info.damageLocation = pos;
		info.regenCooldown = 100;
		m_healthHistory.push(info);

		printf("damage %f\n", info.currentHealth);

		damageCallback(h, info);
	}

	void doRegen(float h)
	{
		if (h == 0)
			return;

		if (!m_healthHistory.empty())
		{
			HealthInfo &info = m_healthHistory.top();
			info.regenCooldown--;
			if (info.regenCooldown >= 0)
				return;
		}

		m_health += h;
		m_health = glm::clamp(m_health, 0.0f, 1.0f);

		while (!m_healthHistory.empty())
		{
			HealthInfo &info = m_healthHistory.top();
			printf("regen %f\n", info.currentHealth);

			if (m_health >= info.currentHealth)
			{
				m_healthHistory.pop();
				regenCallback(info.h, info);
			}
			else break;
		}
	}

private:
	std::stack<HealthInfo> m_healthHistory;
	float m_health = 1.0f; // fully allive = 1.0
};

inline vec2 physCoordToGfx(const vec2& c) {
	return vec2(50, -25) * c.x + vec2(50, 25) * c.y;
}

inline vec2 gfxCoordToPhys(const vec2& c) {
	return vec2(0.01f, 0.01f) * c.x + vec2(-0.02f, 0.02f) * c.y;
}

enum class RaftPartType
{
	Platform,
	Attachment,
	Payload,

	NumItems
};

struct RaftPartBlueprint
{
	RaftPartType type;
	ivec2 size = ivec2(1);
	ivec2 pivot = ivec2(1);
	std::string texture;
	std::string name;
	std::string weapon;
	float regen = 0.0f;
	float thrust = 0.0f;
	float mass = 1.0f;
	float chance = 1.0f;
	bool generate = true;	// should it be generated for random rafts, and as a pickup?
};

// Go around the part and iterate all potential attachment points which aren't already occupied
// Note that this skips corners, as we don't want ugly connections with just tips touching.
template <typename Fn>
void iterRectOutline(const ivec2& size, const ivec2& offset, Fn fn) {
	for (int y = 1; y < size.y - 1; ++y) {
		fn(offset.x, y + offset.y);
		fn(size.x - 1 + offset.x, y + offset.y);
	}

	for (int x = 1; x < size.x - 1; ++x) {
		fn(x + offset.x, offset.y);
		fn(x + offset.x, size.y - 1 + offset.y);
	}
}

struct PlayerInputState
{
	vec2 moveDir = vec2(0.0f);
	vec2 shootTarget = vec2(0.0f);
	bool fire1 = false;
	bool fire2 = false;
	bool action = false;
};
