#pragma once

#include <Framework/Framework.h>
#include <Framework/SoundLibrary.h>
#include <Framework/Math.h>
#include <Box2D/Box2D.h>

#include "Common.h"
#include "WaterWorld.h"

#include <vector>

struct Sprite;

struct Debris : public BoatyBase
{
	ivec2 m_size = ivec2(1, 1);
	ivec2 m_offset = ivec2(0, 0);

	b2Body* m_body = nullptr;

	class DebrisSystem* m_debrisSystem = nullptr;

	vec2 getPhysicalPosition() const {
		return m_body->GetPosition();
	}

	vec2 getRenderPosition() const {
		return physCoordToGfx(m_body->GetPosition()) + g_waterWorld->getObjectWaveSway(m_body->GetPosition());
	}

	void createPhysics(vec2 position);
	
	bool onCollision(BoatyBase* collector);

	Debris(ivec2 size, ivec2 offset, DebrisSystem* system)
		: m_size(size)
		, m_offset(offset)
		, m_debrisSystem(system)
	{}
};

class DebrisSystem
{
public:
	DebrisSystem();
	~DebrisSystem();

	void setSprite(Sprite* sprite) { m_sprite = sprite; }

	void playPickupSound();

	void spawnRandomDebris(int gridSize, int amount, float minimumAllowableDistance);
	void spawnDebris(vec2 aroundPosition, float minimumAllowableDistance);

	Debris* spawnDebrisAtPoint(vec2 point);
	void removeDebris(Debris* debris);

	const std::vector<Debris*>& activeDebris() const;

	Sprite* getSprite() const { return m_sprite; }

private:
	std::vector<Debris*> m_activeDebris;
	Sprite* m_sprite = nullptr;
	SoundClip m_pickupSound;
};