#include "PeckFX.h"
#include <imgui.h>
#include <algorithm>
#include <Framework/Sprite.h>
#include "WaterWorld.h"

float forScience = 1.0f;

float magnitude(vec2 in)
{
	return sqrtf(in.x*in.x + in.y*in.y);
}

vec2 limit(vec2 in, float max)
{
	float size = magnitude(in);
	if (size > max)
	{
		return vec2(in.x / size, in.y / size);
	}
	else
	{
		return in;
	}
}
Peck::Peck(vec2 pos)
{
	//float angle = randomUnitFloat() * M_PI * 2;
	//velocity = vec2(cosf(angle), sinf(angle));

	maxSpeed = 3.5;
	maxForce = 0.5;
	velocity = vec2(rand() % 3 - 2, rand() % 3 - 2);
	position = pos;
}

void Peck::step(std::vector<Peck>& pecks)
{
	flock(pecks);
	update();
	const float width = 1280*2;
	const float height = 720*2;
	borders(width, height);
}

void Peck::applyForce(vec2 force)
{
	// We could add mass here if we want A = F / M
	acceleration += force;
}

void Peck::flock(std::vector<Peck>& pecks)
{
	vec2 sep = separate(pecks);   // Separation
	vec2 ali = align(pecks);      // Alignment
	vec2 coh = cohesion(pecks);   // Cohesion
	vec2 wall = target(pecks);		// Bias towards wall!

	// Arbitrarily weight these forces
	sep *= 1.5f;
	ali *= 1.0f;
	coh *= 1.0f;
	//wall *= 4.0f;

	// Add the force vectors to acceleration
	applyForce(sep);
	applyForce(ali);
	applyForce(coh);
	applyForce(wall);
}

void Peck::update()
{
	acceleration *= 0.35f;

	// Update velocity
	velocity += acceleration;
	
	// Limit speed
	velocity = limit(velocity, maxSpeed);
	position += velocity;
	
	// Reset accelertion to 0 each cycle
	acceleration *= 0;
}

// A method that calculates and applies a steering force towards a target
vec2 Peck::seek(vec2 target)
{
	vec2 desired = target;  // A vector pointing from the position to the target
									   // Scale to maximum speed
	desired = glm::normalize(desired);
	desired *= maxSpeed;

	acceleration = desired - velocity;
	acceleration = limit(acceleration, maxForce); // Limit to maximum steering force

	return acceleration;
}

// Separation
// Method checks for nearby pecks and steers away
vec2 Peck::separate(std::vector<Peck>& pecks)
{
	float desiredSeparation = 20.0f * forScience;
	vec2 steer;
	int count = 0;

	// For every boid in the system, check if it's too close
	for (Peck& other : pecks)
	{
		// Calculate distance from current peck to peck we're looking at
		float d = glm::distance(position, other.position);
		
		// If this is a fellow peck and it's too close, move away from it
		if ((d > 0) && (d < desiredSeparation))
		{
			vec2 diff(0, 0);
			vec2 tmp;
			tmp.x = position.x - other.position.x;
			tmp.y = position.y - other.position.y;
			diff = tmp;
			diff = glm::normalize(diff);
			diff /= d; // Weight by distance
			steer += diff;
			count++;
		}

		// If the distance is greater than 0 and less than an arbitrary amount (0 when you are yourself)
		/*if ((d > 0) && (d < desiredSeparation))
		{
			// Calculate vector pointing away from neighbor
			vec2 diff = glm::normalize(position - other.position);

			diff /= d; // Weight by distance
			steer += diff;
			count++;
		}*/
	}

	// Average
	if (count > 0)
	{
		steer /= (float)count;
	}

	// As long as the vector is greater than 0
	if (magnitude(steer) > 0)
	{
		// Implement Reynolds: Steering = Desired - Velocity
		steer = glm::normalize(steer);
		steer *= maxSpeed;
		steer -= velocity;
		steer = limit(steer, maxForce);
	}

	return steer;
}

// Alignment
// For every nearby peck in the system, calculate the average velocity
vec2 Peck::align(std::vector<Peck>& pecks)
{
	float neighbordist = 70 * forScience;
	vec2 sum;
	int count = 0;
	
	for (Peck& other : pecks)
	{
		float d = glm::distance(position, other.position);
		if ((d > 0) && (d < neighbordist))
		{
			sum += other.velocity;
			count++;
		}
	}

	if (count > 0)
	{
		sum /= (float)count;
		
		// Implement Reynolds: Steering = Desired - Velocity
		sum = glm::normalize(sum);
		sum *= maxSpeed;

		vec2 steer = sum - velocity;
		steer = limit(steer, maxForce);
		return steer;
	}
	
	return vec2(0, 0);
}
static vec2 blah;
// Cohesion
// For the average position (i.e. center) of all nearby boids, calculate steering vector towards that position
vec2 Peck::cohesion(std::vector<Peck>& pecks)
{
	//return vec2(0, 0);
	float neighborDist = 25 * forScience;
	vec2 sum; // Start with empty vector to accumulate all positions
	int count = 0;
	for (Peck& other : pecks)
	{
		float d = glm::distance(position, other.position);
		if ((d > 0) && (d < neighborDist))
		{
			sum += other.position;
			count++;
		}
	}

	if (count > 0)
	{
		sum /= count;
		return seek(sum);  // Steer towards the position
	}
	
	return vec2(0, 0);
}

vec2 Peck::target(std::vector<Peck>& pecks)
{
	//return vec2(0, 0);
	//float neighborDist = 25 * forScience;
	return seek(vec2(60000, 60000) * maxSpeed);
}

int accum = 0;

// Wrap-around
void Peck::borders(float width, float height)
{
	return;
	if (position.x < -radius) position.x =  width + radius;
	if (position.y < -radius) position.y = height + radius;
	if (position.x > width  + radius) position.x = -radius;
	if (position.y > height + radius) position.y = -radius;
}

void Peck::render(PrimitiveBatch* prim)
{
	vec2 renderPosition = position;
	//renderPosition += g_waterWorld->getObjectWaveSway(gfxCoordToPhys(renderPosition));

	aabb2 peckBounds;
	peckBounds.min = renderPosition - (radius);
	peckBounds.max = renderPosition + (radius);
	prim->drawTexturedQuad(peckBounds);
}

void Flock::add(Peck peck)
{
	pecks.push_back(peck);
}

void Flock::step()
{
	for (auto& peck : pecks)
	{
		peck.step(pecks);
	}
}

aabb2 Flock::getBounds()
{
	vec3 boundingCircle; // xy = center, z = radius

	aabb2 boundingBox = aabbExpandStart();

	for (auto& peck : pecks)
	{
		aabbExpand(boundingBox, peck.position);
	}

	//vec2 center = aabbCenter(boundingBox);
	//boundingCircle.x = center.x;
	//boundingCircle.y = center.y;
	//boundingCircle.z = std::max(1.f, std::max(boundingBox.width() * 0.5f, boundingBox.height() * 0.5f));

	return boundingBox;
}

void Flock::render(PrimitiveBatch* prim, Sprite* sprite0, Sprite* sprite1)
{
	accum++;

	prim->setTexture(sprite0->texture);
	for (auto i = 0; i < pecks.size() / 2; ++i)
	{
		pecks[i].render(prim);
	}

	prim->setTexture(sprite1->texture);
	for (auto i = pecks.size() / 2; i < pecks.size(); ++i)
	{
		pecks[i].render(prim);
	}

	prim->flush();
}

PeckFX::PeckFX()
{
}

PeckFX::~PeckFX()
{
}

void PeckFX::step(Flock& flock)
{
	flock.step();
}

void PeckFX::render(PrimitiveBatch* prim, Flock& flock, Sprite* sprite0, Sprite* sprite1)
{
	flock.render(prim, sprite0, sprite1);
}