#pragma once

#include <Framework/Framework.h>
#include <Framework/SoundLibrary.h>
#include <Framework/PrimitiveBatch.h>
#include <Framework/Math.h>
#include <Box2D/Box2D.h>

#include "Common.h"

#include <vector>

struct Sprite;

struct Peck
{
	vec2 position;
	vec2 velocity;
	vec2 acceleration;

	float radius = 16;
	float maxForce = 0.03f;
	float maxSpeed = 0.0125f;

	Peck(vec2 pos);

	void step(std::vector<Peck>& pecks);
	void flock(std::vector<Peck>& pecks);
	void applyForce(vec2 force);

	void update();

	vec2 seek(vec2 target);

	void render(PrimitiveBatch* prim);

	// Factors
	vec2 separate(std::vector<Peck>& pecks);
	vec2 align(std::vector<Peck>& pecks);
	vec2 cohesion(std::vector<Peck>& pecks);
	vec2 target(std::vector<Peck>& pecks);

	void borders(float width, float height);
};

struct Flock
{
	std::vector<Peck> pecks;

	aabb2 getBounds();

	void add(Peck peck);
	void step();
	void render(PrimitiveBatch* prim, Sprite* sprite0, Sprite* sprite1);
};

class PeckFX
{
public:
	PeckFX();
	~PeckFX();

	void step(Flock& flock);
	void render(PrimitiveBatch* prim, Flock& flock, Sprite* sprite0, Sprite* sprite1);
};
