#include "RaftAI.h"
#include "Raft.h"
#include "WaterWorld.h"

#include <iostream>
#include <string>

void RaftAI::enterObserving(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::exitObserving(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::enterExploring(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());

	raftAI->wanderTarget = g_waterWorld->m_playerRaft->getPhysicsPosition() - raftAI->raft->getPhysicsPosition();
	if (length(raftAI->wanderTarget) > 0.01f) {
		raftAI->wanderTarget = glm::normalize(raftAI->wanderTarget);
	}
}

void RaftAI::exitExploring(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::enterCombat(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::exitCombat(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());

	// Record last known player location
	raftAI->lastKnownPlayerPosition = g_waterWorld->m_playerRaft->getPhysicsPosition();
}

void RaftAI::enterPewPew(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::exitPewPew(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::enterRam(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::exitRam(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::enterReverse(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
	remainingMoves = (rand() % 1 + 1) * 60 /*hz*/;
}

void RaftAI::exitReverse(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::enterPatrolling(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());

	raftAI->patrolStart = raftAI->raft->getPhysicsPosition();
	raftAI->patrolFinish = raftAI->patrolStart + glm::normalize(raftAI->lastKnownPlayerPosition - raftAI->patrolStart) * 5.0f;
	raftAI->remainingAttempts = 4; // 4 tries along the path before giving up
	raftAI->remainingMoves = 10 * 60 /*hz*/; // sec
}

void RaftAI::exitPatrolling(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
	raftAI->lastKnownPlayerPosition = vec2();
}

void RaftAI::enterDeath(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::exitDeath(RaftStateMachine::StatePtr state)
{
	auto raftAI = static_cast<RaftAI*>(state->getParent()->getUserData());
}

void RaftAI::debugEventAction(
	RaftStateMachine::State::EventPtr event,
	RaftStateMachine::StatePtr currentState,
	RaftStateMachine::StatePtr nextState)
{
	if (currentState == nextState)
		std::cout << "Action: [" << event->getId() << "] " << "No State Change" << std::endl;
	else
		std::cout << "Action: [" << event->getId() << "] " << currentState->getId() << " --> " << nextState->getId() << std::endl;
}

void RaftAI::onCollision(RaftPart* playerRaftPart)
{
	assert(playerRaftPart);
	if (machine.getCurrentState()->getId() == "ramming")
	{
		// We hit the player! Lets start a cooldown timer, reverse
		// a bit, and then get ready for another pass at them.
		remainingMoves = 30;
		pendingState = "reverse";
		machine.nextState("wait");
	}
}

void initializeRaftAI(Raft* actor, RaftAI& ai)
{
	using namespace std::placeholders;

	ai.machine.reset();
	ai.machine.setUserData(&ai);

	// Observing
	ai.machine.addState("observing")
		->bindEntryAction(std::bind(&RaftAI::enterObserving, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitObserving, &ai, _1));

	ai.machine.addState("observing")->addEvent("explore", "exploring")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("observing")->addEvent("spotted_enemy", "combat")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	// Exploring
	ai.machine.addState("exploring")
		->bindEntryAction(std::bind(&RaftAI::enterExploring, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitExploring, &ai, _1));

	ai.machine.addState("exploring")->addEvent("observe", "observing")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("exploring")->addEvent("spotted_enemy", "combat")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	// Combat
	ai.machine.addState("combat")
		->bindEntryAction(std::bind(&RaftAI::enterCombat, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitCombat, &ai, _1));

	ai.machine.addState("combat")->addEvent("damaged", "combat")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	ai.machine.addState("combat")->addEvent("killed_enemy", "patrolling")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("combat")->addEvent("lost_enemy", "patrolling")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("combat")->addEvent("ram", "ramming")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("combat")->addEvent("pewpew", "pewpew")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	ai.machine.addState("pewpew")
		->bindEntryAction(std::bind(&RaftAI::enterPewPew, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitPewPew, &ai, _1));
	ai.machine.addState("pewpew")->addEvent("patrol", "patrolling")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	ai.machine.addState("ramming")
		->bindEntryAction(std::bind(&RaftAI::enterRam, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitRam, &ai, _1));
	ai.machine.addState("ramming")->addEvent("wait", "wait")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("ramming")->addEvent("patrol", "patrolling")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	ai.machine.addState("reverse")
		->bindEntryAction(std::bind(&RaftAI::enterReverse, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitReverse, &ai, _1));

	ai.machine.addState("reverse")->addEvent("done", "combat")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	ai.machine.addState("wait");// ->addEvent("done", "wait")->bindAction(std::bind(&RaftAI::waitDoneAction, &ai, _1, _2, _3));
	ai.machine.addState("wait")->addEvent("combat", "combat")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("wait")->addEvent("reverse", "reverse")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	// Patrolling
	ai.machine.addState("patrolling")
		->bindEntryAction(std::bind(&RaftAI::enterPatrolling, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitPatrolling, &ai, _1));

	ai.machine.addState("patrolling")->addEvent("relaxed", "patrolling")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("patrolling")->addEvent("alerted", "patrolling")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("patrolling")->addEvent("spotted_enemy", "combat")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("patrolling")->addEvent("explore", "exploring")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("patrolling")->addEvent("observe", "observing")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	// Death
	ai.machine.addState("death")
		->bindEntryAction(std::bind(&RaftAI::enterDeath, &ai, _1))
		->bindExitAction(std::bind(&RaftAI::exitDeath, &ai, _1));

	ai.machine.addState("death")->addEvent("destroying", "death")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));
	ai.machine.addState("death")->addEvent("destroyed", "death")->bindAction(std::bind(&RaftAI::debugEventAction, &ai, _1, _2, _3));

	// Debug
	ai.machine.dumpStates();

	ai.raft = actor;
	ai.enabled = true;

	// graham: Make it interesting, switch to explore instead of observe.
	ai.machine.nextState("explore");
}

bool canSeeEachOther(Raft* a, Raft* b)
{
	assert(a);
	assert(b);
	auto aVis = a->getVisibilityCircle();
	auto bVis = b->getVisibilityCircle();

	vec2 d = vec2(aVis.x - bVis.x, aVis.y - bVis.y);
	float dist = sqrtf(d.x*d.x + d.y*d.y);
	return (dist < aVis.z + bVis.z);
}