#pragma once

#include <Framework/Math.h>

#include "StateMachine.h"

typedef StateMachine<std::string, std::string> RaftStateMachine;

struct RaftAI
{
	bool enabled = false;

	struct Raft* raft = nullptr;

	RaftStateMachine machine;

	void enterObserving(RaftStateMachine::StatePtr state);
	void exitObserving(RaftStateMachine::StatePtr state);

	void enterExploring(RaftStateMachine::StatePtr state);
	void exitExploring(RaftStateMachine::StatePtr state);

	void enterCombat(RaftStateMachine::StatePtr state);
	void exitCombat(RaftStateMachine::StatePtr state);

	void enterPewPew(RaftStateMachine::StatePtr state);
	void exitPewPew(RaftStateMachine::StatePtr state);

	void enterRam(RaftStateMachine::StatePtr state);
	void exitRam(RaftStateMachine::StatePtr state);

	void enterReverse(RaftStateMachine::StatePtr state);
	void exitReverse(RaftStateMachine::StatePtr state);

	void enterPatrolling(RaftStateMachine::StatePtr state);
	void exitPatrolling(RaftStateMachine::StatePtr state);

	void enterDeath(RaftStateMachine::StatePtr state);
	void exitDeath(RaftStateMachine::StatePtr state);

	void debugEventAction(
		RaftStateMachine::State::EventPtr event,
		RaftStateMachine::StatePtr currentState,
		RaftStateMachine::StatePtr nextState);

	void waitDoneAction(
		RaftStateMachine::State::EventPtr event,
		RaftStateMachine::StatePtr currentState,
		RaftStateMachine::StatePtr nextState);

	void onCollision(struct RaftPart* playerRaftPart);

	std::string pendingState;

	// Set during combat
	int remainingMoves = 0;
	int remainingAttempts = 0;

	// Set when exiting combat
	vec2 lastKnownPlayerPosition;

	// Set when entering patrol
	vec2 patrolStart;
	vec2 patrolFinish;

	// explore parameters
	float wanderRadius = 2.0f;
	float wanderDistance = 1.0f;
	float wanderJitter = 0.1f;
	vec2 wanderTarget;
};

void initializeRaftAI(Raft* actor, RaftAI& ai);

bool canSeeEachOther(Raft* a, Raft* b);