#include <Framework/TextureLibrary.h>

#include "RaftGen.h"
#include "RaftPartLibrary.h"
#include "Raft.h"

#include <vector>
#include <set>

/*bool Raft::canAddPart(const ivec2& offset, RaftPartBlueprint* part) const {
	return canAddPart(offset, part->size, Rafter::bit(part->type));
}*/

void Raft::addPart(ivec2 offset, RaftPartBlueprint* part) {
	rasterPart(offset, part->size, Rafter::bit(part->type));

	if (part->type == RaftPartType::Platform) {
		// Rasterize the ID of this platform, so that we can easily find links later
		for (int y = 0; y < part->size.y; ++y) {
			for (int x = 0; x < part->size.x; ++x) {
				rafter.setPlatformId(x + offset.x, y + offset.y, u16(m_parts.size()));
			}
		}
	}

	//m_parts.push_back({ part, offset });
	m_parts.push_back(RaftPart(part, offset, this));
}

void Raft::addPayload(int partIdx, ivec2 offset, RaftPartBlueprint* part) {
	rasterPart(offset, part->size, Rafter::bit(part->type));
	m_parts[partIdx].payload.push_back(RaftPart(part, offset, this));
}


/*void addPart(Raft& raft, RaftPartBlueprint* blueprint, vec2 offset) {
	raft.m_parts.push_back(RaftPart(blueprint->size, offset, &raft));
	raft.m_parts.back().m_sprite = g_textureLibrary->getSprite(blueprint->texture);
}*/

RaftGen::RaftGen(RaftPartLibrary* lib)
	: lib(*lib)
{
	for (auto& it : lib->library) {
		if (it.second.type == RaftPartType::Platform) {
			platforms.push_back(&it.second);
		}

		if (it.second.type == RaftPartType::Attachment) {
			attachments.push_back(&it.second);
		}

		if (it.second.type == RaftPartType::Payload && it.second.generate) {
			payloads.push_back(&it.second);
		}
	}
}

Raft* RaftGen::generateRaft(vec2 position, int targetArea, bool isPlayer) {
	Raft& craft = *new Raft;

	craft.addPart(ivec2(0, 0), platforms[rand() % platforms.size()]);
	craft.addPayload(0, ivec2(0, 0), lib.get("rovivrus"));
	targetArea -= craft.m_parts[0].area();

	for (int i = 0; i < 20 && targetArea > 0; ++i) {
		auto platform = platforms[rand() % platforms.size()];

		std::vector<ivec3> attachmentPoints;
		craft.iterAttachmentPoints(platform->size, [&](ivec2 p, uint partIdx) { attachmentPoints.push_back(ivec3(p, partIdx)); });
		if (0 == attachmentPoints.size()) {
			break;
		}

		ivec3 pt = attachmentPoints[rand() % attachmentPoints.size()];
		craft.addPart(ivec2(pt), platform);
		targetArea -= craft.m_parts.back().area();
	}

	for (int i = 0; i < 4; ++i) {
		auto platform = attachments[rand() % attachments.size()];

		std::vector<ivec3> attachmentPoints;
		craft.iterAttachmentPoints(platform->size, [&](ivec2 p, uint partIdx) { attachmentPoints.push_back(ivec3(p, partIdx)); });
		if (0 == attachmentPoints.size()) {
			break;
		}

		ivec3 pt = attachmentPoints[rand() % attachmentPoints.size()];
		craft.addPart(ivec2(pt), platform);
		targetArea -= craft.m_parts.back().area();
	}

	// Add payload items
	for (int i = 0; i < 4; ++i) {
		auto payload = payloads[rand() % payloads.size()];

		std::vector<ivec3> attachmentPoints;
		craft.iterPayloadPoints(payload->size, [&](ivec2 p, uint partIdx) { attachmentPoints.push_back(ivec3(p, partIdx)); });
		if (0 == attachmentPoints.size()) {
			break;
		}

		ivec3 pt = attachmentPoints[rand() % attachmentPoints.size()];
		craft.addPayload(pt.z, ivec2(pt), payload);
	}

	/*Raft* raftPtr = new Raft;
	Raft& raft = *raftPtr;*/

	// Spawn the correct parts and payloads
	/*for (const RaftCraft::CraftPart &p : craft.parts) {
		raft.m_parts.push_back(RaftPart(p.part->size, p.offset, raftPtr));
		raft.m_parts.back().m_sprite = g_textureLibrary->getSprite(p.part->texture);

		for (const RaftCraft::CraftPart &pload: p.payload) {
			auto& platform = raft.m_parts.back();

			RaftPayload payload;
			payload.m_sprite = g_textureLibrary->getSprite(pload.part->texture);
			payload.m_offset = pload.offset;

			platform.payload.push_back(payload);
		}
	}*/

	craft.findLinks();
	craft.createPhysics(position, isPlayer);

	if (!isPlayer)
		craft.createAI();

	return &craft;
}

Raft* RaftGen::makeTestRaft(vec2 position, bool isPlayer) {
	return generateRaft(position, 12, isPlayer);

	/*Raft* raftPtr = new Raft;
	Raft& raft = *raftPtr;

	addPart(raft, lib.get("door"), vec2(0, 0));
	addPart(raft, lib.get("door"), vec2(1, 0));
	addPart(raft, lib.get("lifebuoy"), vec2(0, 1));
	addPart(raft, lib.get("lifebuoy"), vec2(1, -1));

	raft.link(0, 1);
	raft.link(0, 2);
	raft.link(1, 2);
	raft.link(1, 3);

	{
		RaftPayload payload;
		payload.m_sprite = g_textureLibrary->getSprite("fridge");
		raft.m_parts[0].payload.push_back(payload);
	}

	{
		RaftPayload payload;
		payload.m_sprite = g_textureLibrary->getSprite("lawnmower");
		raft.m_parts[1].payload.push_back(payload);
	}

	raft.createPhysics(position, isPlayer);

	return raftPtr;*/
}
