#include "RaftPartLibrary.h"

#include <Framework/JsonUtil.h>
#include <rapidjson/document.h>

RaftPartLibrary* g_raftPartLibrary = nullptr;

RaftPartLibrary::RaftPartLibrary(const char* filename)
{
	SDL_assert(g_raftPartLibrary == nullptr);
	g_raftPartLibrary = this;

	JsonDocument document = jsonParseFromFile(filename);

	for (auto it = document.MemberBegin(); it != document.MemberEnd(); ++it)
	{
		const char* name = it->name.GetString();
		
		RaftPartBlueprint& blueprint = library[name];
		blueprint.texture = name;
		blueprint.name = name;

		auto obj = it->value.GetObject();
		for (auto it = obj.MemberBegin(); it != obj.MemberEnd(); ++it)
		{
			const char* name = it->name.GetString();
			if (!strcmp(name, "texture"))
			{
				blueprint.texture = it->value.GetString();
			}
			else if (!strcmp(name, "type"))
			{
				const char* value = it->value.GetString();
				blueprint.type = stringToEnumInsensitive(value, RaftPartType::Platform);
			}
			else if (!strcmp(name, "size"))
			{
				auto sizeObj = it->value.GetArray();
				blueprint.size.x = sizeObj[0].GetInt();
				blueprint.size.y = sizeObj[1].GetInt();
			}
			else if (!strcmp(name, "pivot"))
			{
				auto sizeObj = it->value.GetArray();
				blueprint.pivot.x = sizeObj[0].GetInt();
				blueprint.pivot.y = sizeObj[1].GetInt();
			}
			else if (!strcmp(name, "regen"))
			{
				blueprint.regen = it->value.GetFloat();
			}
			else if (!strcmp(name, "thrust"))
			{
				blueprint.thrust = it->value.GetFloat();
			}
			else if (!strcmp(name, "weapon"))
			{
				blueprint.weapon = it->value.GetString();
			}
			else if (!strcmp(name, "mass"))
			{
				blueprint.mass = it->value.GetFloat();
			}
			else if (!strcmp(name, "chance"))
			{
				blueprint.chance = it->value.GetFloat();
			}
			else if (!strcmp(name, "generate"))
			{
				blueprint.generate = it->value.GetBool();
			}
		}
	}

	std::vector<RaftPartBlueprint*> blueprints;
	for (auto& it : library) {
		blueprints.push_back(&it.second);
	}

	weightedLibrary = new WeightedRaftPartLibrary(blueprints);
}

WeightedRaftPartLibrary::WeightedRaftPartLibrary(std::vector<RaftPartBlueprint*> blueprints)
	: gen(unsigned(std::time(0)))
	, items(blueprints)
{
	for (auto bp : items) {
		weights.push_back(double(bp->chance));
	}

	distrib = std::discrete_distribution<int>(std::begin(weights), std::end(weights));
}
