#include "RaftRenderer.h"

#include <Framework/TextureLibrary.h>
#include <Framework/Sprite.h>
#include <Framework/PrimitiveBatch.h>

#include <algorithm>

void Rafterizer::reset()
{
	m_elements.clear();
}

void Rafterizer::add(const Raft& raft)
{
	for(const RaftPart& part : raft.m_parts)
	{
		if (!part.m_sprite) {
			continue;
		}

		for (const RaftPart& p : part.payload) {
			Element elem;
			elem.sprite = p.m_sprite;

			ivec2 pivot = p.m_blueprint->pivot;
			elem.pos =
				part.getRenderPosition()
				+ physCoordToGfx(0.5f * vec2(-part.size() + p.size()))
				- vec2(pivot.x, p.m_sprite->h - pivot.y - 1)
				+ physCoordToGfx(p.m_offset - part.m_offset)
				;

			elem.sortPos = elem.pos.y - 666;
			elem.obj = &p;
			m_elements.push_back(elem);
		}

		Element elem;
		elem.sprite = part.m_sprite;
		elem.pos = part.getRenderPosition() - vec2(part.m_sprite->w, part.m_sprite->h) * 0.5f;
		elem.sortPos = elem.pos.y;
		elem.obj = &part;
		m_elements.push_back(elem);
	}
}

void Rafterizer::add(const Debris& debris, Sprite* sprite)
{
	assert(sprite);
	
	Element elem;
	elem.sprite = sprite;
	elem.pos = debris.getRenderPosition() - vec2(sprite->w, sprite->h) * 0.5f;
	elem.sortPos = elem.pos.y;
	elem.obj = &debris;

	m_elements.push_back(elem);
}

void Rafterizer::sortElements()
{
	std::sort(m_elements.begin(), m_elements.end(), 
		[](const Element& a, const Element& b)
	{
		if (a.sortPos == b.sortPos) {
			return (uintptr_t)a.sprite < (uintptr_t)b.sprite;
		}
		else {
			return a.sortPos > b.sortPos;
		}
	});
}

bool needsHealthBar(const Rafterizer::Element& elem)
{
	const RaftPart* p = dynamic_cast<const RaftPart*>(elem.obj);
	return p && p->m_blueprint->type != RaftPartType::Payload;
}

void Rafterizer::render()
{
	sortElements();

	for (const Element& elem : m_elements)
	{
		drawSprite(g_primitiveRenderer, *elem.sprite, elem.pos);
	}

	if (m_enableHealthBars) {

		vec2 healthBarSize = vec2(40, 5);

		for (const Element& elem : m_elements)
		{
			if (needsHealthBar(elem)) {
				aabb2 rect = { elem.pos, elem.pos + vec2(healthBarSize.x * elem.obj->getHealth(), healthBarSize.y) };
				g_primitiveRenderer->drawRect(rect, ColorRGBA8::Green());
			}
		}

		ColorRGBA8 outlineColor = ColorRGBA8::Black();

		g_primitiveRenderer->setLineWidth(1.0);
		for (const Element& elem : m_elements)
		{
			if (needsHealthBar(elem)) {
				vec2 _y = vec2(0, healthBarSize.y);
				vec2 x_ = vec2(healthBarSize.x, 0);
				vec2 xy = vec2(healthBarSize.x, healthBarSize.y);

				g_primitiveRenderer->drawLine(elem.pos, elem.pos + _y, outlineColor);
				g_primitiveRenderer->drawLine(elem.pos, elem.pos + x_, outlineColor);
				g_primitiveRenderer->drawLine(elem.pos + _y, elem.pos + xy, outlineColor);
				g_primitiveRenderer->drawLine(elem.pos + x_, elem.pos + xy, outlineColor);
			}
		}

	}
}

