#pragma once

#include <Framework/Framework.h>
#include <Framework/Math.h>
#include <Framework/Color.h>
#include <vector>

#include "Raft.h"

class BitmapFontRenderer;
struct Sprite;
struct PlayerInputState;

struct RaftPartSlot
{
	ivec2 localPosition = ivec2(0);
	int parentPartIndex = 0;
	vec2 physicalPosition = vec2(0.0f);

	RaftPartSlot(ivec2 _localPosition, int _parentPartIndex, vec2 _physicalPosition)
	: localPosition(_localPosition)
	, parentPartIndex(_parentPartIndex)
	, physicalPosition(_physicalPosition)
	{
	}
};

class RaftUI
{
public:

	RaftUI();
	~RaftUI();

	void update(const PlayerInputState& input);

	void selectRaft(Raft* raft);

	void renderWorldSpace(); // draw world-space UI (markers, health, icons)
	void renderScreenSpace(); // draw screens-space UI (text, menu, etc)

	void renderAvailableParts();
	void renderMessage();

	void renderKeepOrTrash();

	void renderSelectAttachmentPoint();
	void renderSelectAttachmentPointWS();

	bool wantPauseSimulation() const;

	Raft* m_selectedRaft = nullptr;
	BitmapFontRenderer* m_fontRegular;
	BitmapFontRenderer* m_fontLarge;

	bool m_showAvailableParts = false;
	bool m_showMessage = false;
	bool m_showselectAttachmentPoint = false;

	PlayerInputState m_lastInputState;

	struct MessageInfo
	{
		ColorRGBA8 captionColor = ColorRGBA8(255, 200, 50);
		ColorRGBA8 textColor = ColorRGBA8(255, 255, 255);
		std::string caption = "Game Over!";
		std::string text = "Press any key to start again.";
		const Sprite* sprite;
	};
	MessageInfo m_message;
	
	struct StoryMessage
	{
		std::string text;
		const Sprite* sprite = nullptr;
	};
	std::vector<StoryMessage> m_storyMessages;

	void renderStoryMessage();

	bool m_showGameOver = false;
	void renderGameOver();

private:

	struct MenuItem
	{
		std::string partName;
		const Sprite* sprite;
		std::string text;
		vec2 size;
	};

	void updateAvailablePartsMenu();
	void dimWorld(u8 alpha = 200);

	std::vector<MenuItem> m_menuItems;
	int m_selectedMenuItem = 0;
	int m_cursorMoveCooldown = 0;

	std::string m_selectedPartName;
	std::vector<RaftPartSlot> m_availablePartSlots;
	int m_selectedAttachmentSlot = 0;

	Raft m_tempRaft;
};

extern RaftUI* g_raftUI;

