#pragma once

#include <Framework/Framework.h>
#include <Framework/Graphics.h>
#include <Framework/PrimitiveBatch.h>
#include <Framework/Math.h>
#include <Framework/Color.h>
#include <Framework/Sprite.h>

struct WaterGrid
{
	ColorRGBA particleColor = ColorRGBA(0.0f, 1.0f, 0.0f);

	float particleSpeed = 0.008f;
	float particleXDelay = 0.281f;
	float particleYDelay = 0.150f;
	float particleRadius = 3.0f;
	float particleOrbitDiameter = 23.0f;
	int distanceBetweenOrbits = 76;

    unsigned int matrixSize = 0;

	float uvTiling = 0.070f;
    
    // WIP
    // tangent = vec4(1f, 0f, 0f, -1f);
    std::vector<vec2> vertices;
    std::vector<int> triangles;
};

class WaterSimulator
{
public:
	WaterSimulator();
	~WaterSimulator();

	void init(WaterGrid& grid, int gridSize);
	void step(WaterGrid& grid);
	void edit(WaterGrid& grid);
	float evalHeightAtPoint(WaterGrid& grid, vec2 pt);
    
    int getTimeIndex() const { return m_timeIndex; }
    
private:
    int m_timeIndex = 0;
};

class WaterRenderer
{
public:
	WaterRenderer();
	~WaterRenderer();

	void renderControls(PrimitiveBatch* prim, WaterGrid& grid, int timeIndex);
    void renderMesh(PrimitiveBatch* prim, WaterGrid& grid, Sprite* image, int timeIndex, vec2 gridOffset, float zoom);
};
