#pragma once

#include <Framework/Framework.h>
#include <Framework/Graphics.h>
#include <Framework/SoundLibrary.h>
#include "WeaponSystem.h"
#include "ParticleSystem.h"
#include "PeckFX.h"

struct Raft;
struct RaftPartBlueprint;
struct Debris;
struct Sprite;
class Rafterizer;
class CameraShake;
class DebrisSystem;

struct WorldUpdateInfo
{
	float dt;
	PlayerInputState playerInputState;
	CameraShake *shake;
};

class WaterSimulator;
struct WaterGrid;
struct RaftGen;

class WaterWorld
{
public:
	WaterWorld();

	void spawnRafts();

	void update(aabb2 viewBounds, const WorldUpdateInfo& info);

	void render(Rafterizer& rafterizer, DebrisSystem& debrisSystem);
	void renderFlocks(aabb2 viewBounds, PrimitiveBatch* prim, Sprite* sprite0, Sprite* sprite1);

	void addRaft(Raft* raft, bool isPlayer);

	void collectPart(RaftPartBlueprint* blueprint);

	vec2 getObjectWaveSway(vec2 pos) const;

	void playSmashSound();
	void doneSmashSound();
	
	std::vector<Raft*> m_rafts;
	Raft* m_playerRaft = nullptr;
	WeaponSystem m_weaponSystem;
	ParticleSystem particles;
	WaterSimulator* m_waterSimulator = nullptr;
	WaterGrid* m_waterGrid = nullptr;
	RaftGen* m_raftGen;
	
	const int flockCount = 50;
	std::vector<Flock> m_flocks;
	PeckFX m_peckFX;

	std::vector<RaftPartBlueprint*> m_unattachedParts;
	u32 m_unattachedPartsVersion = 0;

	SoundClip m_smashSound;
	bool m_smashPlaying = false;
};

extern WaterWorld* g_waterWorld;
