#pragma once
#include "Animation.h"
#include "Common.h"
#include "Framework/PhysicsSystem.h"

class GfxContext;


struct BulletInfo
{
	int numRays = 10;
	float blastPower = 50.0f;
	float dampening = 5.0f;
	float size = 0.05f;
	float density = 60.0f;
	float restitution = 0.99f;
	float damage = 0.2f;
	bool shotByPlayer = true;
	vec2 center;
};

class Bullet : public BoatyBase
{
public:
	BulletInfo info;
	b2Body *body;
	b2Fixture *fixture;
	AnimationSet *animationSet;
};

class Explosion
{
public:
	float angle;
	vec2 center;
	AnimationSet *animationSet;
};

class WeaponSystem
{
public:
	Bullet bullets[1 << 20];
	int bulletCount = 0;

	Explosion explosions[1 << 20];
	int explosionCount = 0;
	
	void spawnExplosion(vec2 center);
	void shotgun(vec2 center, vec2 dir, float spread, int bulletCount, bool isPlayer);
	void grenade(vec2 center);
	void update(float dt);
	void draw(GfxContext* ctx, PrimitiveBatch* prim);

private:
	void explode(BulletInfo info, std::function<b2Vec2(int, int)> rayGenerator);
};