#pragma once

#include "Framework.h"
#include "PrimitiveBatch.h"

class Stream;

// Class for working with AngelCode BMFont files.
// More details here: http://www.angelcode.com/products/bmfont/
// Current simple implementation is only suitable for ASCI single-byte encoding.
class BitmapFont
{
public:

	enum { MaxFontPages = 16, InvalidPage = 0xFF };

	struct CharData
	{
		u16 x, y, width, height;
		s16 offset_x, offset_y, advance_x;
		u8 page, chan;
	};

	struct PageData
	{
		char filename[128];
	};

	BitmapFont();

	bool read(Stream& stream);

	const CharData&	getChar(u8 c) const { return m_chars[c]; }
	const char*		getPage(u32 n) const { return m_pages[n].filename; }
	u32				getPageCount() const { return m_num_pages; }

	u32				getSize() const { return m_size; }

private:

	CharData	m_chars[256];
	PageData	m_pages[MaxFontPages];
	u32			m_num_pages;
	u32			m_size;
};

struct BitmapFontData
{
	const void* header_data;
	size_t header_size;
	const void* pixels_data;
	size_t pixels_size;
	u32 width;
	u32 height;
	GfxFormat format;
};

class BitmapFontRenderer
{
public:

	BitmapFontRenderer(const BitmapFontData& data);
	BitmapFontRenderer(const void* header_data, size_t header_size, const void* pixels_data, size_t pixels_size, u32 width, u32 height, GfxFormat format);
	BitmapFontRenderer(const char* font_filename, const char* texture_path);
	~BitmapFontRenderer();

	// returns position after the last drawn character
	vec2 drawString(PrimitiveBatch* batch, const vec2& pos, const char* str, ColorRGBA8 col = ColorRGBA8::White(), bool flush = true);

	// returns the width and height of the text block in pixels
	vec2 measureString(const char* str);

private:

	void init(const void* header_data, size_t header_size, const void* pixels_data, size_t pixels_size, u32 width, u32 height, GfxFormat format);
	void createQuads();

private:

	BitmapFont m_font_data;
	std::vector<TextureHandle> m_textures;
	std::vector<TextureDescr> m_texture_descr;
	TexturedQuad2D m_chars[256];
};


