#include "Color.h"

float epsilon = 0.00001f;

ColorRGBA::operator ColorRGBA8() const
{

	u8 r8 = u8(saturate(r) * 255.0f);
	u8 g8 = u8(saturate(g) * 255.0f);
	u8 b8 = u8(saturate(b) * 255.0f);
	u8 a8 = u8(saturate(a) * 255.0f);

	return ColorRGBA8(r8, g8, b8, a8);

}

void ColorRGBA::rgb_to_hsv()
{
	float h = 0;
	float s = 0;
	float v = 0;

	float min, max, delta;
	min = glm::min(r, glm::min(g, b));
	max = glm::max(r, glm::max(g, b));
	v = max;				// v


	delta = max - min;
	if (max > epsilon)
		s = delta / max;		// s
	else {
		// r = g = b = 0		// s = 0, v is undefined
		// s = 0;
		// h = -1;
		return;
	}
	if (fabs(r - max) < epsilon)
		h = (g - b) / delta;		// between yellow & magenta
	else if (fabs(g - max) < epsilon)
		h = 2 + (b - r) / delta;	// between cyan & yellow
	else
		h = 4 + (r - g) / delta;	// between magenta & cyan
	h *= 60;				// degrees
	if (h < 0)
		h += 360;

	r = h;
	g = s;
	b = v;
}

void ColorRGBA::hsv_to_rgb()
{
	float h = r;
	float s = g;
	float v = b;

	int i;
	float f, p, q, t;
	if (s < epsilon) // achromatic (grey) 
	{
		r = g = b;
		return;
	}

	h /= 60;			// sector 0 to 5
	i = int(floor(h));
	f = h - i;			// factorial part of h
	p = v * (1 - s);
	q = v * (1 - s * f);
	t = v * (1 - s * (1 - f));
	switch (i) {
	case 0:
		r = v;
		g = t;
		b = p;
		break;
	case 1:
		r = q;
		g = v;
		b = p;
		break;
	case 2:
		r = p;
		g = v;
		b = t;
		break;
	case 3:
		r = p;
		g = q;
		b = v;
		break;
	case 4:
		r = t;
		g = p;
		b = v;
		break;
	default:		// case 5:
		r = v;
		g = p;
		b = q;
		break;
	}
}
