#pragma once

#include "Framework.h"
#include "Math.h"

struct ColorRGBA;
struct ColorRGBA8;

// high-precision (32 bit floating point) color
struct ColorRGBA
{
	float r, g, b, a;

	ColorRGBA(){}

	ColorRGBA(float _r, float _g, float _b, float _a = 1.0f)
		: r(_r)
		, g(_g)
		, b(_b)
		, a(_a)
	{
	}

	explicit ColorRGBA(vec3 vec)
		: r(vec.x)
		, g(vec.y)
		, b(vec.z)
		, a(1.0f)
	{
	}

	explicit ColorRGBA(const vec4 &vec)
		: r(vec.x)
		, g(vec.y)
		, b(vec.z)
		, a(vec.w)
	{
	}

	void rgb_to_hsv();
	void hsv_to_rgb();

	operator vec4() const
	{
		return vec4(r, g, b, a);
	}

	operator ColorRGBA8() const;

	//////////////////////////////////////////////////////////////////////////

	static ColorRGBA Black()	{ return ColorRGBA(0.0f, 0.0f, 0.0f); }
	static ColorRGBA White()	{ return ColorRGBA(1.0f, 1.0f, 1.0f); }
	static ColorRGBA Red()		{ return ColorRGBA(1.0f, 0.0f, 0.0f); }
	static ColorRGBA Green()	{ return ColorRGBA(0.0f, 1.0f, 0.0f); }
	static ColorRGBA Blue()	{ return ColorRGBA(0.0f, 0.0f, 1.0f); }
	static ColorRGBA Cyan()	{ return ColorRGBA(0.0f, 1.0f, 1.0f); }
	static ColorRGBA Magenta()	{ return ColorRGBA(1.0f, 0.0f, 1.0f); }
	static ColorRGBA Yellow()	{ return ColorRGBA(1.0f, 1.0f, 0.0f); }
	static ColorRGBA Orange()	{ return ColorRGBA(1.0f, 0.5f, 0.0f); }
	static ColorRGBA Purple()	{ return ColorRGBA(0.5f, 0.0f, 0.5f); }
};


// low-precision (8 bit unsigned integer) color
struct ColorRGBA8
{
	u8 r, g, b, a;

	ColorRGBA8(){}

	explicit ColorRGBA8(u32 col)
	{
		*this = *reinterpret_cast<ColorRGBA8*>(&col);
	}

	ColorRGBA8(u8 _r, u8 _g, u8 _b, u8 _a = 255)
		: r(_r)
		, g(_g)
		, b(_b)
		, a(_a)
	{
	}

	operator ColorRGBA() const
	{
		float rf = float(r) / 255.0f;
		float gf = float(g) / 255.0f;
		float bf = float(b) / 255.0f;
		float af = float(a) / 255.0f;

		return ColorRGBA(rf, gf, bf, af);
	}

	operator u32() const
	{
		return *reinterpret_cast<const u32*>(this);
	}

	static  ColorRGBA8 Black(u8 alpha=255)	{ return ColorRGBA8(0x00, 0x00, 0x00, alpha); }
	static  ColorRGBA8 White(u8 alpha = 255)	{ return ColorRGBA8(0xFF, 0xFF, 0xFF, alpha); }
	static  ColorRGBA8 Red(u8 alpha = 255)	{ return ColorRGBA8(0xFF, 0x00, 0x00, alpha); }
	static  ColorRGBA8 Green(u8 alpha = 255)	{ return ColorRGBA8(0x00, 0xFF, 0x00, alpha); }
	static  ColorRGBA8 Blue(u8 alpha = 255)	{ return ColorRGBA8(0x00, 0x00, 0xFF, alpha); }
	static  ColorRGBA8 Yellow(u8 alpha = 255) { return ColorRGBA8(0xFF, 0xFF, 0x00, alpha); }

};

inline float linear_to_srgb(float v)
{
	v = glm::min(1.0f, v);
	return v <= 0.0031308f ? v * 12.92f : 1.055f * pow(v, 1.0f / 2.4f) - 0.055f;
}

inline ColorRGBA linear_to_srgb(ColorRGBA col)
{
	col.r = linear_to_srgb(col.r);
	col.g = linear_to_srgb(col.g);
	col.b = linear_to_srgb(col.b);

	return col;
}
inline vec3 linear_to_srgb(vec3 col)
{
	col.x = linear_to_srgb(col.x);
	col.y = linear_to_srgb(col.y);
	col.z = linear_to_srgb(col.z);

	return col;
}
