#pragma once

#include "Framework.h"
#include "Sprite.h"

#include <vector>

struct ReplayStream;
struct RenderContext;
struct cJSON;
struct StateMod;
class b2Body;
class JammyWorld;
class TextureLibrary;

enum class Allegiance { // TODO: move to separate file?
	Player,
	Enemy,
	Neutral,
	NumItems
};

class GameObject
{
public:
	cJSON *					json = nullptr;
	std::vector<StateMod*>	stateMods;
	Allegiance				allegiance = Allegiance::Neutral;
	bool					alive = true;
	int						hp = 100;
	bool					paused = false;
	bool					hideWeapon = false;
	const char *			type = nullptr;
	vec2					position = vec2(0.0f);
	b2Body *				m_body = nullptr;

	virtual ~GameObject() {};

	virtual void update(float dt, JammyWorld* world) {}
	virtual void updateBehavior(float dt, JammyWorld* world) {}
	virtual void create(b2Body *body, TextureLibrary * library, cJSON *prefab, cJSON *object) = 0;
	virtual void draw(RenderContext& rc, RenderMode mode) {}
	virtual void dumpReplayState(ReplayStream& stream) const {}
	virtual void loadReplayState(ReplayStream& stream) {}
	virtual void suppressChat(float duration) {}
	void addStateMod(StateMod*);

	vec2 getPhysicsPosition() const {
		return position;
	}

	aabb2 getPhysicsBounds() const;

	vec2 getRenderPosition() const;

	template <typename T>
	void basicReflect(T& r)
	{
		BASIC_REFLECT_MEMBER(r, alive);
		BASIC_REFLECT_MEMBER(r, hp);
		BASIC_REFLECT_MEMBER(r, paused);
		BASIC_REFLECT_MEMBER(r, hideWeapon);
		BASIC_REFLECT_MEMBER(r, position);
	}
};
