#pragma once

#include "Framework.h"

#include <cJSON.h>

namespace rapidjson
{
class CrtAllocator;
template <typename BaseAllocator> class MemoryPoolAllocator;
template<typename CharType> struct UTF8;
template <typename Encoding, typename Allocator> class GenericValue;
template <typename Encoding, typename Allocator, typename StackAllocator> class GenericDocument;
template <bool Const, typename ValueT> class GenericObject;
typedef GenericDocument<UTF8<char>, MemoryPoolAllocator<CrtAllocator>, CrtAllocator> Document;
typedef GenericValue<UTF8<char>, MemoryPoolAllocator<CrtAllocator>> Value;
typedef GenericObject<false, Value> Object;
typedef GenericObject<true, Value> ConstObject;
}

bool jsonGetValue(cJSON* obj, const char* name, int& out);
bool jsonGetValue(cJSON* obj, const char* name, float& out);
bool jsonGetValue(cJSON* obj, const char* name, std::string& out);

inline void jsonSetIntValue(cJSON* object, int val) // warning-free replacement for cJSON_SetIntValue
{
	if (object)
	{
		object->valueint = val;
		object->valuedouble = (float)val;
	}
}

template <typename T> bool jsonGetValue(const T& obj, const char* name, int& out) {
	if (obj[name].IsInt()) {
		out = obj[name].GetInt();
		return true;
	}
	else {
		return false;
	}
}

template <typename T> bool jsonGetValue(const T& obj, const char* name, float& out) {
	if (obj[name].IsFloat()) {
		out = obj[name].GetFloat();
		return true;
	}
	else {
		return false;
	}
};
template <typename T> bool jsonGetValue(const T& obj, const char* name, std::string& out) {
	if (obj[name].IsString()) {
		out = obj[name].GetString();
		return true;
	}
	else {
		return false;
	}
}

typedef rapidjson::Document JsonDocument;
typedef rapidjson::Value JsonValue;

JsonDocument jsonParseFromFile(const char* filename);
const char* jsonGetParseError(const JsonDocument& document);
