#pragma once

#include "Framework.h"
#include <glm/glm.hpp>

using glm::ivec2;
using glm::ivec3;
using glm::ivec4;

using glm::vec2;
using glm::vec3;
using glm::vec4;
using glm::mat4x4;

__forceinline float saturate(float val)
{
	return glm::min(1.0f, glm::max(0.0f, val));
}

template <typename T>
__forceinline T lerp(const T& a, const T& b, float alpha)
{
	return a*(1.0f - alpha) + b*alpha;
}

struct line2
{
	vec2 a;
	vec2 b;
};

struct line3
{
	vec3 a;
	vec3 b;
	vec3 c;
};

struct ray2
{
	vec2 o;
	vec2 d;
};

struct ray3
{
	vec3 o;
	vec3 d;
};

struct aabb2
{
	vec2 min;
	vec2 max;

	vec2 tl() const { return vec2(min.x, max.y); }
	vec2 tr() const { return vec2(max.x, max.y); }
	vec2 bl() const { return vec2(min.x, min.y); }
	vec2 br() const { return vec2(max.x, min.y); }

	float width()  const { return max.x - min.x; }
	float height() const { return max.y - min.y; }

	vec2 dimensions() const { return max - min; }

	void translate(const vec2& offset) {
		min += offset;
		max += offset;
	}
};

struct aabb3
{
	vec3 min;
	vec3 max;
};

struct circle
{
	vec2 center;
	float radius;
};

struct sphere
{
	vec3 center;
	float radius;
};

// math helpers

inline vec2 aabbCenter(aabb2 box)
{
	return (box.max + box.min) * 0.5f;
}

inline aabb2 aabbExpandStart()
{
	aabb2 box;
	box.min = vec2(FLT_MAX);
	box.max = vec2(-FLT_MAX);
	return box;
}

inline void aabbExpand(aabb2& box, vec2 pos)
{
	box.min.x = glm::min(box.min.x, pos.x);
	box.min.y = glm::min(box.min.y, pos.y);
	box.max.x = glm::max(box.max.x, pos.x);
	box.max.y = glm::max(box.max.y, pos.y);
}

inline aabb2 aabbUnion(aabb2 boxA, aabb2 boxB)
{
	aabb2 box;
	box.min = glm::min(boxA.min, boxB.min);
	box.max = glm::max(boxA.max, boxB.max);
	return box;
}

inline aabb2 aabbTranslate(aabb2 box, vec2 deltaPos)
{
	box.min += deltaPos;
	box.max += deltaPos;
	return box;
}

float signedTriArea(vec2 a, vec2 b, vec2 c);

bool intersectPointBox(vec2 p, aabb2 box);
bool intersectLineLine(line2 lineA, line2 lineB, vec2* outPos = nullptr);
bool intersectBoxBox(aabb2 boxA, aabb2 boxB);

