#pragma once

#include "Framework.h"
#include "Physics.h"
#include "PrimitiveBatch.h"
#include <glm/mat2x2.hpp>
#include <vector>

using glm::mat2;


class PhysicsDebugDraw : public b2Draw
{
public:
	float worldScale;
	PrimitiveBatch* prim;
	vec2 offset;
	mat2 transform = mat2(1, 0, 0, 1);

	PhysicsDebugDraw(float worldScale)
		: worldScale(worldScale)
	{
		SetFlags(~0);
	}

	/// Draw a closed polygon provided in CCW order.
	virtual void DrawPolygon(const b2Vec2* vertices, int32 vertexCount, const b2Color& color)
	{
		std::vector<SDL_Point> lines;

		for (int32 i = 0; i < vertexCount; i++)
		{
			SDL_Point p;

			vec2 vertex = vertices[i];
			vertex = transform * vertex;

			p.x = (int)((vertex.x * worldScale) + offset.x);
			p.y = (int)((vertex.y * worldScale) + offset.y);

			lines.push_back(p);
		}

		prim->setTexture(InvalidResourceHandle());
		PrimitiveBatch::BatchVertex* v = prim->drawVertices(PrimitiveType::LineLoop, (u32)lines.size());

		ColorRGBA8 gfxColor((Uint8)(color.r * 255), (Uint8)(color.g * 255), (Uint8)(color.b * 255));
		int idx = 0;
		for(SDL_Point p : lines)
		{
			v[idx].pos.x = (float)p.x;
			v[idx].pos.y = (float)p.y;
			v[idx].pos.z = 0;
			v[idx].tex.x = 0;
			v[idx].tex.y = 0;
			v[idx].col0 = gfxColor;

			idx++;
		}
	}

	/// Draw a solid closed polygon provided in CCW order.
	virtual void DrawSolidPolygon(const b2Vec2* vertices, int32 vertexCount, const b2Color& color)
	{
		DrawPolygon(vertices, vertexCount, color);
	}

	/// Draw a circle.
	virtual void DrawCircle(const b2Vec2& center, float32 radius, const b2Color& color)
	{
		const int num = 32;
		b2Vec2 a = center + b2Vec2(0.0f, radius);
		for (int i = 1; i <= num; ++i)
		{
			float t = 2.0f* b2_pi * float(i) / num;
			b2Vec2 b = center + b2Vec2(radius * sin(t), radius * cos(t));
			DrawSegment(a, b, color);
			a = b;
		}
	}

	/// Draw a solid circle.
	virtual void DrawSolidCircle(const b2Vec2& center, float32 radius, const b2Vec2& axis, const b2Color& color)
	{
		DrawCircle(center, radius, color);
		DrawSegment(center, center+radius*axis, color);
	}

	/// Draw a line segment.
	virtual void DrawSegment(const b2Vec2& p1, const b2Vec2& p2, const b2Color& color)
	{
		b2Vec2 v[] = { p1, p2 };
		DrawPolygon(v, 2, color);
	}

	/// Draw a transform. Choose your own length scale.
	/// @param xf a transform.
	virtual void DrawTransform(const b2Transform& xf)
	{
		int x = 0;

	}

};