#include <Framework/Framework.h>
#include <Framework/PhysicsDebugDraw.h>
#include <set>
#include "PhysicsSystem.h"

PhysicsSystem* g_physicsSystem = nullptr;


PhysicsSystem::PhysicsSystem(const vec2& gravity, float physicsWorldScale)
{
	physicsWorld = new b2World(gravity);
	physicsDebugDraw = new PhysicsDebugDraw(physicsWorldScale);
	physicsWorld->SetDebugDraw(physicsDebugDraw);
}

void PhysicsSystem::handleCollision(std::function<void(b2Contact *, std::set<b2Body*>&)> onContact, std::function<void(b2Body*)> onDestroy)
{
	std::set<b2Body*> destroy;

	b2Contact *contact = physicsWorld->GetContactList();
	while (contact)
	{
		onContact(contact, destroy);

		contact = contact->GetNext();
	}

	for (b2Body *body : destroy)
	{
		onDestroy(body);
		physicsWorld->DestroyBody(body);
	}
}

b2Body* createPhysicsBody(const ColliderDesc& colliderDesc, const vec2& offset, unsigned short mask, unsigned short category) {
	b2BodyDef body;
	body.position.Set(offset.x, offset.y);
	body.bullet = false;
	body.type = b2_dynamicBody;
	body.fixedRotation = true;
	b2Body* result = g_physicsSystem->physicsWorld->CreateBody(&body);

	b2FixtureDef fixture;
	fixture.density = 1.0f;//this->density;
	fixture.friction = 0.1f;//this->friction;
	fixture.restitution = 0.5f;//this->restitution;
	fixture.filter.maskBits = mask;
	fixture.filter.categoryBits = 3;

	if (colliderDesc.type == ColliderDesc::Type::Box)
	{
		b2PolygonShape *shape = new b2PolygonShape;
		shape->SetAsBox(std::max(1e-3f, colliderDesc.box.width), std::max(1e-3f, colliderDesc.box.height));
		fixture.shape = shape;
	}
	else if (colliderDesc.type == ColliderDesc::Type::Circle)
	{
		float radius = std::max(1e-3f, colliderDesc.circle.radius);
		b2CircleShape *shape = new b2CircleShape;
		shape->m_radius = radius;
		fixture.shape = shape;
	}

	result->CreateFixture(&fixture);
	return result;
}
