#pragma once

#include "Reflection.h"
#include <Box2D/Box2D.h>

#include <set>

struct ColliderDesc {
	enum class Type {
		Box,
		Circle
	} type;

	union {
		struct {
			float width;
			float height;
		} box;

		struct {
			float radius;
		} circle;
	};

	static ColliderDesc makeBox(float w, float h) {
		ColliderDesc result;
		result.type = Type::Box;
		result.box.width = w;
		result.box.height = h;
		return result;
	}

	static ColliderDesc makeCircle(float r) {
		ColliderDesc result;
		result.type = Type::Circle;
		result.circle.radius = r;
		return result;
	}

	bool operator==(const ColliderDesc& other) const {
		if (type != other.type) return false;
		if (Type::Box == type) {
			return box.width == other.box.width && box.height == other.box.height;
		}
		else {
			return circle.radius == other.circle.radius;
		}
	}

	ColliderDesc() : type(Type::Circle)
	{
		circle.radius = 10.0f;
		box.height = 10.0f;
	}
};

/*template<> inline cJSON* jsonMarshall(const ColliderDesc& src) {
	cJSON* res = cJSON_CreateObject();
	cJSON_AddStringToObject(res, "type", (src.type == ColliderDesc::Type::Box ? "box" : "circle"));
	if (src.type == ColliderDesc::Type::Box) {
		cJSON_AddNumberToObject(res, "width", src.box.width);
		cJSON_AddNumberToObject(res, "height", src.box.height);
	}
	else {
		cJSON_AddNumberToObject(res, "radius", src.circle.radius);
	}
	return res;
}
template<> inline void jsonDemarshall(const cJSON* json, ColliderDesc* dst) {
	assert(json->type == cJSON_Object);
	// TODO
}*/

class PhysicsDebugDraw;

struct PhysicsSystem {
	PhysicsSystem(const vec2& gravity, float physicsWorldScale);
	void handleCollision(std::function<void(b2Contact *, std::set<b2Body*>&)> onContact, std::function<void(b2Body*)> onDestroy = [](b2Body*) {});

	b2World *physicsWorld = nullptr;
	PhysicsDebugDraw *physicsDebugDraw = nullptr;
} extern *g_physicsSystem;

b2Body* createPhysicsBody(const ColliderDesc& colliderDesc, const vec2& offset, unsigned short mask = 0xffff, unsigned short category = 1);
