#pragma once

#include <Framework/Framework.h>
#include <Framework/Graphics.h>

#include <vector>
#include <functional>

struct SDL_Texture;
struct SDL_Renderer;
struct SDL_Rect;

struct SDLSprite
{
	SDL_Texture *texture;
	int tw, th;
	int sw, sh;
};

struct Sprite {
	TextureHandle			texture;
	std::vector<ColorRGBA8>	pixels;
	glm::ivec2				offset = glm::ivec2(0, 0);
	glm::ivec2				origSize = glm::ivec2(0, 0);
	int						w = 0;
	int						h = 0;
};

void loadSpriteData(Sprite *const sprite, const char *textureFile, const u32 colorKey, bool clip, bool repeat);
Sprite* loadSprite(const char *filename, const u32 colorKey, bool clip, bool repeat, const std::function<void(Sprite* const)>& onReload);
Sprite* loadSprite(const char *filename, const u32 colorKey, bool clip, bool repeat);
Sprite* cloneSprite(Sprite *const sprite);
u8 linearRgbToSrgbByte(float x);

template<typename Closure>
Sprite* createSprite(u32 w, u32 h, const Closure& fn) {
	Sprite *const sprite = new Sprite;
	// g_loadedSprites.push_back(sprite); // Yuriy: this is undefined here

	std::vector<ColorRGBA8>& pixels = sprite->pixels;
	pixels.resize(w * h);

	for (u32 y = 0; y < h; ++y) {
		for (u32 x = 0; x < w; ++x) {
			glm::vec4 px = fn(glm::vec2(float(x) / (w - 1), float(y) / (h - 1)));
			pixels[y * w + x] = ColorRGBA8(
				linearRgbToSrgbByte(px.x),
				linearRgbToSrgbByte(px.y),
				linearRgbToSrgbByte(px.z),
				u8(std::min(std::max(0.0f, px.w), 1.0f) * 255)
			);
		}
	}

	TextureDescr texDescr = TextureDescr::create_2D(w, h, GfxFormat_RGBA8_Unorm);
	sprite->texture = Gfx_Texture_Create(g_device, texDescr, pixels.data(), w * h * 4);
	sprite->w = w;
	sprite->h = h;

	return sprite;
}

void drawSprite(SDLSprite *s, SDL_Renderer *renderer, int idx, SDL_Rect *dstrect);
void print(SDLSprite *s, SDL_Renderer *renderer, const char *str, int x, int y, int k = 3);

void drawSprite(PrimitiveBatch *const prim, const Sprite& sprite, const vec2& pos, float angle = 0);

extern uint PixelSize;

struct RenderContext {
	vec2			offset = vec2(0, 0);
	PrimitiveBatch*	prim = nullptr;
};

enum class RenderMode {
	Default,
	Shadow,
	Selected,
};

