#include <Framework/Framework.h>
#include <Framework/FileWatcher.h>
#include <stb_image.h>
#include "SpriteAsset.h"
#include "Marshall.h"

#if 0
SpritePool g_spritePool;


void loadSpriteData(SpriteAsset::Render *const render, SpriteAsset::Info *const info, const SpriteAsset::Url& url)
{
	int width, height, components;

	u8 *const srcPixels = stbi_load(url.filename.c_str(), &width, &height, &components, 4);
	SDL_assert(srcPixels && "Can't load texture");

	if (srcPixels && width > 0 && height > 0)
	{
		if (url.colorKey)
		{
			const u32 numPixels = width * height;
			for (u32 i = 0; i < numPixels; ++i)
			{
				u8& r = srcPixels[i * 4 + 0];
				u8& g = srcPixels[i * 4 + 1];
				u8& b = srcPixels[i * 4 + 2];
				u8& a = srcPixels[i * 4 + 3];

				if (((u32(r) << 16u) + (u32(g) << 8u) + u32(b)) == url.colorKey) {
					a = 0u;
				}
			}
		}

		int minX = width;
		int maxX = 0;
		int minY = height;
		int maxY = 0;

		for (int y = 0; y < height; ++y) {
			for (int x = 0; x < width; ++x) {
				// NOTE: Flipped Y so that the minY will refer to coords after flipping the image.
				const u8 a = srcPixels[((height - y - 1) * width + x) * 4 + 3];
				if (a != 0) {
					minX = std::min(minX, x);
					maxX = std::max(maxX, x);
					minY = std::min(minY, y);
					maxY = std::max(maxY, y);
				}
			}
		}

		if (minX == height) {
			minX = 0;
			maxX = 0;
			minY = 0;
			maxY = 0;
		}

		if (!url.clip) {
			minX = 0;
			maxX = width - 1;
			minY = 0;
			maxY = height - 1;
		}

		info->pixels.resize((maxX - minX + 1) * (maxY - minY + 1));
		render->offset = glm::ivec2(minX, minY);

		render->w = (maxX - minX + 1);
		render->h = (maxY - minY + 1);
		info->origSize = glm::ivec2(width, height);

		// Flip the image and copy the final pixels
		for (int y = 0; y < render->h; ++y) {
			memcpy(
				info->pixels.data() + y * render->w,
				srcPixels + ((height - y - 1 - render->offset.y) * width + render->offset.x) * 4,
				render->w * 4
			);
		}

		TextureDescr texDescr = TextureDescr::create_2D(render->w, render->h, GfxFormat_RGBA8_Unorm);
		TextureHandle tex = Gfx_Texture_Create(g_device, texDescr, info->pixels.data(), render->w * render->h * 4);

		render->texture = tex;
	}

	stbi_image_free(srcPixels);
}

AssetHandle<SpriteAsset> SpritePool::load(const SpriteAsset::Url& url) {
	SpriteAsset::Render render;
	SpriteAsset::Info info;

	loadSpriteData(&render, &info, url);
	sprites.push_back(url, render, info);
	const u32 id = u32(sprites.size() - 1);

	FileWatcher::watchFile(url.filename.c_str(), [this, id, url] {
		loadSpriteData(
			&sprites.get<SpriteAsset::Render>(id),
			&sprites.get<SpriteAsset::Info>(id),
			url
		);
	});

	return AssetHandle<SpriteAsset>::create(id);
}

cJSON* SpritePool::marshall() const {
	cJSON* spriteAssets = cJSON_CreateArray();
	for (size_t i = 0; i < sprites.size(); ++i) {
		cJSON_AddItemToArray(
			spriteAssets,
			marshallObjectFields(&sprites.get<SpriteAsset::Url>(i), SpriteAsset::Url::info())
		);
	}
	return spriteAssets;
}

void SpritePool::demarshall(cJSON* json) {
	int size = cJSON_GetArraySize(json);
	sprites.clear();

	for (int i = 0; i < size; ++i) {
		cJSON* itemJson = cJSON_GetArrayItem(json, i);
		SpriteAsset::Url url;
		demarshallObjectFields(itemJson, &url);
		load(url);
	}
}
#endif
