#include "Framework.h"
#include "StateMod.h"
#include "SoundLibrary.h"
#include "GameObject.h"

struct StateModPtrPrioritySorter {
	bool operator()(StateMod* a, StateMod* b) {
		return a->priority > b->priority;
	}
};

struct StateModPtrLifeSorter {
	bool operator()(StateMod* a, StateMod* b) {
		return a->lifeRemaining > b->lifeRemaining;
	}
};


void updateStateMods(GameObject *const obj, const float dt) {
	for (size_t i = 0, m = obj->stateMods.size(); i < m; ++i) {
		obj->stateMods[i]->process(obj);
		obj->stateMods[i]->lifeRemaining -= dt;
	}

	std::sort(obj->stateMods.begin(), obj->stateMods.end(), StateModPtrLifeSorter());

	size_t numAliveEffects = obj->stateMods.size();
	for (size_t i = 0, m = obj->stateMods.size(); i < m; ++i) {
		if (obj->stateMods[i]->lifeRemaining <= 0.f) {
			numAliveEffects = i;
			break;
		}
	}

	for (size_t i = numAliveEffects, m = obj->stateMods.size(); i < m; ++i) {
		SDL_assert(obj->stateMods[i]->lifeRemaining <= 0.f);
		obj->stateMods[i]->finalize(obj);
	}

	obj->stateMods.resize(numAliveEffects);

	std::sort(obj->stateMods.begin(), obj->stateMods.end(), StateModPtrPrioritySorter());
}


void DyingStateMod::apply(GameObject *const obj) {
	// TODO: take this from the dying anim
	lifeRemaining = 0.1f;
	//critter->stunned = true;
}

void DyingStateMod::finalize(GameObject *const obj) {
	obj->alive = false;
	jamPlaySound(g_sounds["KnightDeath"]);
}


DamageStateMod::DamageStateMod(int amount_, bool playSound_)
	: StateMod()
{
	lifeRemaining = 0.1f;
	amount = amount_;
	playSound = playSound_;
}

void DamageStateMod::apply(GameObject *const obj) {
	const bool wasAlive = obj->hp > 0;

	obj->hp -= amount;
	
	if (obj->alive && playSound)
	{
		jamPlaySound(g_sounds["SwordHitMetal"]);
	}
	
	if (wasAlive && obj->hp <= 0) {
		obj->addStateMod(new DyingStateMod);
	}
}

void DamageStateMod::modifyColorOverlay(vec4 *const col) {
	if (lifeRemaining > 0.f) {
		*col = vec4(1, 0, 0, 1);
	}
}
