#pragma once

#include "Framework.h"

template<typename T, size_t CAPACITY>
struct StaticArray
{
	__forceinline StaticArray()
		: current_size(0)
	{}

	__forceinline T& operator[] (size_t i)
	{
		return data[i];
	}

	__forceinline const T& operator[](size_t i) const
	{
		return data[i];
	}

	__forceinline const T& back() const
	{
		return data[size()-1];
	}

	__forceinline T& back()
	{
		return data[size()-1];
	}

	__forceinline bool push_back(const T& val)
	{
		if( current_size < CAPACITY )
		{
			data[current_size] = val;
			++current_size;
			return true;
		}
		else
		{
			return false;
		}
	}

	__forceinline bool pop_back()
	{
		if( current_size > 0 )
		{
			--current_size;
			return true;
		}
		else
		{
			return false;
		}
	}

	__forceinline void push_back_unsafe(const T& val)
	{
		data[current_size] = val;
		++current_size;
	}

	__forceinline void pop_back_unsafe(const T& val)
	{
		--current_size;
	}

	__forceinline void clear()
	{
		current_size = 0;
	}

	__forceinline size_t capacity() const { return CAPACITY; }
	__forceinline size_t size() const { return current_size; }
	__forceinline size_t element_size() const { return sizeof(T); }

	__forceinline T* begin()
	{
		return data;
	}

	__forceinline T* end()
	{
		return data + current_size;
	}

	T data[CAPACITY];
	size_t current_size;
};

