#include "Stream.h"

#include <string.h>
#include <algorithm>

u32 MemStream::tell()
{
	return m_pos;
}

void MemStream::seek( u32 pos )
{
	m_pos = std::min(pos, m_size);
}

void MemStream::skip( s32 distance )
{
	u32 pos2 = m_pos+distance;
	pos2 = std::min(pos2, m_size);
	m_pos = pos2;
}

void MemStream::rewind()
{
	m_pos = 0;
}

bool MemStream::valid()
{
	return true;
}

u32 MemStream::write( const void* buf, u32 size )
{
	u32 pos2 = m_pos+size;
	pos2 = std::min(pos2, m_size);
	size = pos2-m_pos;
	
	memcpy(&m_data_rw[m_pos], buf, size);
	m_pos = pos2;

	return size;
}

u32 MemStream::read( void* buf, u32 size )
{

	u32 pos2 = m_pos+size;
	pos2 = std::min(pos2, m_size);
	size = pos2 - m_pos;

	memcpy(buf, &m_data_ro[m_pos], size);
	m_pos = pos2;

	return size;
}

MemStream::MemStream(void* data, u32 size)
	: m_data_rw((u8*)data)
	, m_data_ro((u8*)data)
	, m_pos(0)
	, m_size(size)
{
}

MemStream::MemStream(const void* data, u32 size)
	: m_data_rw(nullptr)
	, m_data_ro((const u8*)data)
	, m_pos(0)
	, m_size(size)
{
}

	u32 FileIn::read( void* buf, u32 size )
{
	u32 br = (u32)fread(buf, 1, size, m_file);
	return br;
}

FileIn::FileIn( const char* filename )
{
	m_file = fopen(filename, "rb");
}

FileIn::~FileIn()
{
	if( m_file )
	{
		fclose(m_file);
		m_file = nullptr;
	}
}

u32 FileIn::length()
{
	u32 curr_pos = tell();

	fseek(m_file, 0, SEEK_END);

	u32 end_pos = tell();

	fseek(m_file, curr_pos, SEEK_SET);

	return end_pos;
}

u32 FileOut::write(const void* buf, u32 size)
{
	u32 in_buf_pos	 = 0;
	u32 in_bytes_left = size;

	while(in_bytes_left!=0)
	{
		u32 out_bytes_left = m_buffer_size - m_buffer_pos;

		u32 bytes_to_copy = std::min(in_bytes_left, out_bytes_left);

		memcpy(&m_buffer[m_buffer_pos], &((char*)buf)[in_buf_pos], bytes_to_copy);

		m_buffer_pos	+= bytes_to_copy;
		in_buf_pos		+= bytes_to_copy;

		in_bytes_left	-= bytes_to_copy;

		if( in_bytes_left!=0 || out_bytes_left==0)
		{
			flush();
		}
	}
	return size;
}

FileOut::FileOut( const char* filename, u32 buffer_size ):
	m_buffer(nullptr),
	m_buffer_size(buffer_size),
	m_buffer_pos(0)
{
	m_file = fopen(filename, "wb");
	m_buffer = new char[m_buffer_size];
}

FileOut::~FileOut()
{
	if( m_file ) 
	{
		close();
	}
}

void FileOut::flush()
{
	fwrite(m_buffer, 1, m_buffer_pos, m_file);
	m_buffer_pos = 0;		
}

void FileOut::close()
{
	flush();

	fclose(m_file);
	m_file = nullptr;
	
	delete m_buffer;
	m_buffer = nullptr;
}

u32 FileBase::tell()
{
	return (u32)ftell(m_file); // TODO: large files, maybe?
}

void FileBase::seek( u32 pos )
{
	fseek(m_file, pos, SEEK_SET);
}

void FileBase::skip( s32 distance )
{
	fseek(m_file, distance, SEEK_CUR);
}

void FileBase::rewind()
{
	fseek(m_file, 0, SEEK_SET);
}

bool FileBase::valid()
{
	return m_file != nullptr;
}
