#include "TextureLibrary.h"
#include "Sprite.h"

#include <Framework/JsonUtil.h>
#include <rapidjson/document.h>

TextureLibrary* g_textureLibrary = nullptr;

void TextureLibrary::load(const char *filename)
{
	printf("Loading textures ...\n");

	JsonDocument document = jsonParseFromFile(filename);

	this->filename = filename;

	if (document.HasParseError())
	{
		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Texture library failed to load: %s", jsonGetParseError(document));
		SDL_assert(0 && "Failed to load texture library index");
	}
	else
	{
		for (auto it = document.MemberBegin(); it != document.MemberEnd(); ++it)
		{
			const char* name = it->name.GetString();
			const char* value = it->value.GetString();
			library[name] = loadSprite(value, defaultColorKey, defaultClip, defaultRepeat);
			filenames[name] = value;
		}
	}

	printf("Textures loaded!\n");
}

const char *TextureLibrary::getFilename(const char *name)
{
	return filenames[name].c_str();
}

void TextureLibrary::reload()
{
	load(filename.c_str());
}

bool TextureLibrary::hasSprite(const char *name)
{
	return library[name] != nullptr;
}

Sprite *TextureLibrary::getSprite(const char *name)
{
	SDL_assert(library[name] != nullptr && "Can't find texture in library");
	return library[name];
}

Sprite * TextureLibrary::getSprite(const std::string& name)
{
	SDL_assert(library[name] != nullptr && "Can't find texture in library");
	return library[name];
}
